@@ -436,11 +436,10 @@ static int set_deepest_cpu_idle_state(unsigned int cpu, unsigned int deepest_sta
 
 static inline int have_libcpupower_support(void) { return 1; }
 #else
-static inline int save_cpu_idle_disable_state(__attribute__((unused)) unsigned int cpu) { return -1; }
-static inline int restore_cpu_idle_disable_state(__attribute__((unused)) unsigned int cpu) { return -1; }
+static inline int save_cpu_idle_disable_state(unsigned int cpu) { return -1; }
+static inline int restore_cpu_idle_disable_state(unsigned int cpu) { return -1; }
 static inline void free_cpu_idle_disable_states(void) { }
-static inline int set_deepest_cpu_idle_state(__attribute__((unused)) unsigned int cpu,
-											 __attribute__((unused)) unsigned int state) { return -1; }
+static inline int set_deepest_cpu_idle_state(unsigned int cpu, unsigned int state) { return -1; }
 static inline int have_libcpupower_support(void) { return 0; }
 #endif /* HAVE_LIBCPUPOWER_SUPPORT */
 
@@ -853,6 +852,7 @@ static void *timerthread(void *param)
 				break_thread_id = stat->tid;
 				tracemark("hit latency threshold (%llu > %d)",
 					  (unsigned long long) diff, tracelimit);
+				tracing_stop();
 				break_thread_value = diff;
 			}
 			pthread_mutex_unlock(&break_thread_id_lock);
@@ -1147,7 +1147,7 @@ static void process_options(int argc, char *argv[], int max_cpus)
 			{"deepest-idle-state", required_argument,	NULL, OPT_DEEPEST_IDLE_STATE },
 			{NULL, 0, NULL, 0 },
 		};
-		int c = getopt_long(argc, argv, "a::A::b:c:d:D:F:h:H:i:l:MNo:p:mqrRsSt::uvD:x",
+		int c = getopt_long(argc, argv, "a::A::b:c:d:D:F:h:H:i:l:MNo:p:mqrRsSt::uvx",
 				    long_options, &option_index);
 		if (c == -1)
 			break;
@@ -1687,7 +1687,7 @@ static void rstat_print_stat(struct thread_param *par, int index, int verbose, i
  * thread that creates a named fifo and hands out run stats when someone
  * reads from the fifo.
  */
-static void *fifothread(void *param __attribute__ ((unused)))
+static void *fifothread(void *param)
 {
 	int ret;
 	int fd;
@@ -1868,7 +1868,7 @@ rstat_err:
 	return;
 }
 
-static void write_stats(FILE *f, void *data __attribute__ ((unused)))
+static void write_stats(FILE *f, void *data)
 {
 	struct thread_param **par = parameters;
 	int i;
