@@ -32,8 +32,7 @@
 #define  MAX_COMMAND_LINE 4096
 #define  MAX_TS_SIZE 64
 
-static char debugfileprefix[MAX_PATH];
-static char *fileprefix;
+static char tracefs_prefix[MAX_PATH];
 static int trace_fd = -1;
 static int tracemark_fd = -1;
 static __thread char tracebuf[TRACEBUFSIZ];
@@ -41,28 +40,21 @@ static char test_cmdline[MAX_COMMAND_LINE];
 static char ts_start[MAX_TS_SIZE];
 
 /*
- * Finds the tracing directory in a mounted debugfs
+ * Finds the tracing directory
  */
-char *get_debugfileprefix(void)
+char *get_tracefs_prefix(void)
 {
 	char type[100];
 	FILE *fp;
-	int size;
 	int found = 0;
 	struct stat s;
 
-	if (debugfileprefix[0] != '\0')
+	if (tracefs_prefix[0] != '\0')
 		goto out;
 
 	/* look in the "standard" mount point first */
-	if ((stat("/sys/kernel/debug/tracing", &s) == 0) && S_ISDIR(s.st_mode)) {
-		strcpy(debugfileprefix, "/sys/kernel/debug/tracing/");
-		goto out;
-	}
-
-	/* now look in the "other standard" place */
-	if ((stat("/debug/tracing", &s) == 0) && S_ISDIR(s.st_mode)) {
-		strcpy(debugfileprefix, "/debug/tracing/");
+	if ((stat("/sys/kernel/tracing", &s) == 0) && S_ISDIR(s.st_mode)) {
+		strcpy(tracefs_prefix, "/sys/kernel/tracing/");
 		goto out;
 	}
 
@@ -73,14 +65,8 @@ char *get_debugfileprefix(void)
 	while (fscanf(fp, "%*s %"
 		      STR(MAX_PATH)
 		      "s %99s %*s %*d %*d\n",
-		      debugfileprefix, type) == 2) {
-		if (strcmp(type, "debugfs") == 0) {
-			found = 1;
-			break;
-		}
-		/* stupid check for systemd-style autofs mount */
-		if ((strcmp(debugfileprefix, "/sys/kernel/debug") == 0) &&
-		    (strcmp(type, "systemd") == 0)) {
+		      tracefs_prefix, type) == 2) {
+		if (strcmp(type, "tracefs") == 0) {
 			found = 1;
 			break;
 		}
@@ -88,41 +74,11 @@ char *get_debugfileprefix(void)
 	fclose(fp);
 
 	if (!found) {
-		debugfileprefix[0] = '\0';
+		tracefs_prefix[0] = '\0';
 		goto out;
 	}
-
-	size = sizeof(debugfileprefix) - strlen(debugfileprefix);
-	strncat(debugfileprefix, "/tracing/", size);
-
 out:
-	return debugfileprefix;
-}
-
-int mount_debugfs(char *path)
-{
-	char *mountpoint = path;
-	char cmd[MAX_PATH];
-	char *prefix;
-	int ret;
-
-	/* if it's already mounted just return */
-	prefix = get_debugfileprefix();
-	if (strlen(prefix) != 0) {
-		info("debugfs mountpoint: %s\n", prefix);
-		return 0;
-	}
-	if (!mountpoint)
-		mountpoint = "/sys/kernel/debug";
-
-	sprintf(cmd, "mount -t debugfs debugfs %s", mountpoint);
-	ret = system(cmd);
-	if (ret != 0) {
-		fprintf(stderr, "Error mounting debugfs at %s: %s\n",
-			mountpoint, strerror(errno));
-		return -1;
-	}
-	return 0;
+	return tracefs_prefix;
 }
 
 static char **tracer_list;
@@ -139,7 +95,7 @@ int get_tracers(char ***list)
 	int ret;
 	FILE *fp;
 	char buffer[CHUNKSZ];
-	char *prefix = get_debugfileprefix();
+	char *prefix = get_tracefs_prefix();
 	char *tmpbuf = NULL;
 	char *ptr;
 	int tmpsz = 0;
@@ -219,7 +175,7 @@ int valid_tracer(char *tracername)
  */
 int setevent(char *event, char *val)
 {
-	char *prefix = get_debugfileprefix();
+	char *prefix = get_tracefs_prefix();
 	char buffer[MAX_PATH];
 	int fd;
 	int ret;
@@ -382,12 +338,15 @@ int parse_mem_string(char *str, uint64_t *val)
 	case 'g':
 	case 'G':
 		v *= 1024;
+		__attribute__ ((fallthrough));
 	case 'm':
 	case 'M':
 		v *= 1024;
+		__attribute__ ((fallthrough));
 	case 'k':
 	case 'K':
 		v *= 1024;
+		__attribute__ ((fallthrough));
 	case 'b':
 	case 'B':
 		break;
@@ -403,12 +362,17 @@ int parse_mem_string(char *str, uint64_t *val)
 static void open_tracemark_fd(void)
 {
 	char path[MAX_PATH];
+	int n;
 
 	/*
 	 * open the tracemark file if it's not already open
 	 */
 	if (tracemark_fd < 0) {
-		sprintf(path, "%s/%s", fileprefix, "trace_marker");
+		n = snprintf(path, sizeof(path), "%s/%s", tracefs_prefix, "trace_marker");
+		if (n >= sizeof(path)) {
+			warn("tracefs path too long\n");
+			return;
+		}
 		tracemark_fd = open(path, O_WRONLY);
 		if (tracemark_fd < 0) {
 			warn("unable to open trace_marker file: %s\n", path);
@@ -422,7 +386,11 @@ static void open_tracemark_fd(void)
 	 * if we hit a breaktrace threshold
 	 */
 	if (trace_fd < 0) {
-		sprintf(path, "%s/%s", fileprefix, "tracing_on");
+		n = snprintf(path, sizeof(path), "%s/%s", tracefs_prefix, "tracing_on");
+		if (n >= sizeof(path)) {
+			warn("tracefs path too long\n");
+			return;
+		}
 		if ((trace_fd = open(path, O_WRONLY)) < 0)
 			warn("unable to open tracing_on file: %s\n", path);
 	}
@@ -440,32 +408,27 @@ static void close_tracemark_fd(void)
 static int trace_file_exists(char *name)
 {
 	struct stat sbuf;
-	char *tracing_prefix = get_debugfileprefix();
-	char path[MAX_PATH];
-	strcat(strcpy(path, tracing_prefix), name);
-	return stat(path, &sbuf) ? 0 : 1;
+	return stat(tracefs_prefix, &sbuf) ? 0 : 1;
 }
 
-static void debugfs_prepare(void)
+static void tracefs_prepare(void)
 {
-	if (mount_debugfs(NULL))
-		fatal("could not mount debugfs");
-
-	fileprefix = get_debugfileprefix();
+	get_tracefs_prefix();
 	if (!trace_file_exists("tracing_enabled") &&
 	    !trace_file_exists("tracing_on"))
 		warn("tracing_enabled or tracing_on not found\n"
-		     "debug fs not mounted");
+		     "tracefs not found");
 }
 
 void tracemark(char *fmt, ...)
 {
 	va_list ap;
 	int len;
+	int ret;
 
 	/* bail out if we're not tracing */
 	/* or if the kernel doesn't support trace_mark */
-	if (tracemark_fd < 0 || trace_fd < 0)
+	if (tracemark_fd < 0)
 		return;
 
 	va_start(ap, fmt);
@@ -473,15 +436,25 @@ void tracemark(char *fmt, ...)
 	va_end(ap);
 
 	/* write the tracemark message */
-	write(tracemark_fd, tracebuf, len);
+	ret = write(tracemark_fd, tracebuf, len);
+	if (ret != len)
+		warn("trace mark write failed\n");
+}
 
-	/* now stop any trace */
-	write(trace_fd, "0\n", 2);
+void tracing_stop(void)
+{
+	int ret;
+
+	if (trace_fd < 0)
+		return;
+	ret = write(trace_fd, "0\n", 2);
+	if (ret != 2)
+		warn("trace stop write failed\n");
 }
 
 void enable_trace_mark(void)
 {
-	debugfs_prepare();
+	tracefs_prepare();
 	open_tracemark_fd();
 }
 
