@@ -220,7 +220,7 @@ struct global {
 
 static struct global g;
 
-static void workload_nop(char *dst __attribute__ ((unused)), char *src __attribute__ ((unused)), size_t size __attribute__ ((unused)))
+static void workload_nop(char *dst, char *src, size_t size)
 {
 	/* Nop */
 }
@@ -334,12 +334,13 @@ static void insert_bucket(struct thread *t, stamp_t value)
 	uint64_t extra;
 	double us;
 
-	lat = (value * g.unit_per_us + t->counter_mhz - 1) / t->counter_mhz;
+	lat = (value * g.unit_per_us) / t->counter_mhz;
 	us = (double)lat / g.unit_per_us;
 	if (!g.preheat && g.trace_threshold && us >= g.trace_threshold) {
 		char *line = "%s: Trace threshold (%d us) triggered on cpu %d with %.*f us!\n";
 		tracemark(line, g.app_name, g.trace_threshold, t->core_i,
 			  g.precision, us);
+		tracing_stop();
 		err_quit(line, g.app_name, g.trace_threshold, t->core_i,
 			 g.precision, us);
 	}
@@ -501,7 +502,7 @@ static void write_summary(struct thread *t)
 	calculate(t);
 
 	putfield("Core", t[i].core_i, "d", "");
-	putfield("Counter Freq", t[i].counter_mhz, "u", " (Mhz)");
+	putfield("Counter Freq", t[i].counter_mhz, "u", " (MHz)");
 
 	for (j = 0; j < g.bucket_size; j++) {
 		if (j < g.bucket_size-1 && g.output_omit_zero_buckets) {
@@ -596,7 +597,7 @@ static void run_expt(struct thread *threads, int runtime_secs, bool preheat)
 		pthread_join(threads[i].thread_id, NULL);
 }
 
-static void handle_alarm(int code __attribute__ ((unused)))
+static void handle_alarm(int code)
 {
 	g.cmd = STOP;
 }
