@@ -62,6 +62,7 @@ void *semathread(void *param)
 	cpu_set_t mask;
 	int policy = SCHED_FIFO;
 	struct sched_param schedp;
+	int ret;
 
 	memset(&schedp, 0, sizeof(schedp));
 	schedp.sched_priority = par->priority;
@@ -107,12 +108,15 @@ void *semathread(void *param)
 			if (par->tracelimit && par->maxdiff > par->tracelimit) {
 				char tracing_enabled_file[MAX_PATH];
 
-				strcpy(tracing_enabled_file, get_debugfileprefix());
+				strcpy(tracing_enabled_file, get_tracefs_prefix());
 				strcat(tracing_enabled_file, "tracing_on");
 				int tracing_enabled =
 				    open(tracing_enabled_file, O_WRONLY);
 				if (tracing_enabled >= 0) {
-					write(tracing_enabled, "0", 1);
+					ret = write(tracing_enabled, "0", 1);
+					if (ret < 0)
+						fatal("Could not write to %s: %s\n",
+						      tracing_enabled_file, strerror(errno));
 					close(tracing_enabled);
 				} else
 					fatal("Could not access %s\n",
@@ -321,14 +325,13 @@ static void process_options(int argc, char *argv[])
 
 static int volatile shutdown;
 
-static void sighand(int sig __attribute__ ((unused)))
+static void sighand(int sig)
 {
 	shutdown = 1;
 }
 
-static void print_stat(FILE *fp __attribute__ ((unused)),
-		       struct params *receiver, struct params *sender,
-		       int verbose __attribute__ ((unused)), int quiet)
+static void print_stat(FILE *fp, struct params *receiver, struct params *sender,
+		       int verbose, int quiet)
 {
 	int i;
 
