@@ -33,6 +33,7 @@
 #include "rt-utils.h"
 #include "rt-sched.h"
 #include "rt-error.h"
+#include "histogram.h"
 
 #define _STR(x) #x
 #define STR(x) _STR(x)
@@ -40,6 +41,8 @@
 #define MAXPATH 1024
 #endif
 
+#define HIST_MAX	1000000
+
 #define CPUSET_ALL	"my_cpuset_all"
 #define CPUSET_LOCAL	"my_cpuset"
 
@@ -50,22 +53,16 @@ typedef int s32;
 /* Struct for statistics */
 struct thread_stat {
 	unsigned long cycles;
-	unsigned long cyclesread;
 	long min;
 	long max;
 	long act;
 	double avg;
 	long *values;
-	long *hist_array;
-	long *outliers;
+	struct histogram *hist;
 	pthread_t thread;
 	int threadstarted;
 	int tid;
 	long reduce;
-	long redmax;
-	long cycleofmax;
-	long hist_overflow;
-	long num_outliers;
 };
 
 struct sched_data {
@@ -75,8 +72,6 @@ struct sched_data {
 	int bufmsk;
 
 	struct thread_stat stat;
-
-	char buff[BUFSIZ+1];
 };
 
 static int shutdown;
@@ -84,6 +79,8 @@ static int info_enable;
 static int debug_enable;
 static int tracelimit;
 static int trace_marker;
+static int histogram;
+static FILE *histfile;
 static pthread_mutex_t break_thread_id_lock = PTHREAD_MUTEX_INITIALIZER;
 static pid_t break_thread_id;
 static uint64_t break_thread_value;
@@ -93,10 +90,11 @@ static int cpu_count;
 static int all_cpus;
 static int nr_threads;
 static int use_nsecs;
-static int mark_fd;
 static int quiet;
 static char jsonfile[MAX_PATH];
 
+static struct histoset hset;
+
 static int find_mount(const char *mount, char *debugfs)
 {
 	char type[100];
@@ -135,107 +133,6 @@ static const char *find_debugfs(void)
 	return debugfs;
 }
 
-static int my_vsprintf(char *buf, int size, const char *fmt, va_list ap)
-{
-	const char *p;
-	char tmp[100];
-	char *s = buf;
-	char *end = buf + size;
-	char *str;
-	long long lng;
-	int l;
-	int i;
-
-	end[-1] = 0;
-
-	for (p = fmt; *p && s < end; p++) {
-		if (*p == '%') {
-			l = 0;
- again:
-			p++;
-			switch (*p) {
-			case 's':
-				if (l)
-					fatal("Illegal print format l used with %%s\n");
-				str = va_arg(ap, char *);
-				l = strlen(str);
-				strncpy(s, str, end - s);
-				s += l;
-				break;
-			case 'l':
-				l++;
-				goto again;
-			case 'd':
-				if (l == 1) {
-					if (sizeof(long) == 8)
-						l = 2;
-				}
-				if (l == 2)
-					lng = va_arg(ap, long long);
-				else if (l > 2)
-					fatal("Illegal print format l=%d\n", l);
-				else
-					lng = va_arg(ap, int);
-				i = 0;
-				while (lng > 0) {
-					tmp[i++] = (lng % 10) + '0';
-					lng /= 10;
-				}
-				tmp[i] = 0;
-				l = strlen(tmp);
-				if (!l) {
-					*s++ = '0';
-				} else {
-					while (l)
-						*s++ = tmp[--l];
-				}
-				break;
-			default:
-				fatal("Illegal print format '%c'\n", *p);
-			}
-			continue;
-		}
-		*s++ = *p;
-	}
-
-	return s - buf;
-}
-
-static void ftrace_write(char *buf, const char *fmt, ...)
-{
-	va_list ap;
-	int n;
-
-	if (mark_fd < 0)
-		return;
-
-	va_start(ap, fmt);
-	n = my_vsprintf(buf, BUFSIZ, fmt, ap);
-	va_end(ap);
-
-	write(mark_fd, buf, n);
-}
-
-static void setup_ftrace_marker(void)
-{
-	struct stat st;
-	const char *debugfs = find_debugfs();
-	char files[strlen(debugfs) + strlen("/tracing/trace_marker") + 1];
-	int ret;
-
-	if (strlen(debugfs) == 0)
-		return;
-
-	sprintf(files, "%s/tracing/trace_marker", debugfs);
-	ret = stat(files, &st);
-	if (ret >= 0)
-		goto found;
-	/* Do nothing if not mounted */
-	return;
-found:
-	mark_fd = open(files, O_WRONLY);
-}
-
 /*
  * Return true if file exists
  */
@@ -263,20 +160,15 @@ static int fill_sched_features(char *path)
 	if (strlen(debugfs) == 0)
 		return -1;
 
-	snprintf(path, MAX_PATH, "%s/sched/features", debugfs);
-	ret = check_file_exists(path);
-	if (ret)
-		return 0;
-
-	snprintf(path, MAX_PATH, "%s/sched_features", debugfs);
+	ret = snprintf(path, MAX_PATH, "%s/sched/features", debugfs);
+	if (ret >= MAX_PATH)
+		return -1;
 	ret = check_file_exists(path);
 	if (ret)
 		return 0;
 
 	memset(path, 0, MAX_PATH);
-
 	return ret;
-
 }
 
 static int setup_hr_tick(void)
@@ -447,6 +339,10 @@ static int mount_cpuset(void)
 	if (fd < 0)
 		return fd;
 	ret = write(fd, "0", 2);
+	if (ret < 0) {
+		close(fd);
+		return ret;
+	}
 	close(fd);
 
 	return 0;
@@ -633,7 +529,14 @@ static void destroy_cpuset(const char *name, int print)
 		sprintf(buf, "%d", pid);
 		if (print)
 			printf("Moving %d out of %s\n", pid, name);
-		write(fd, buf, strlen(buf));
+		ret = write(fd, buf, strlen(buf));
+		if (ret < 0 && errno == ENOSPC) {
+			fclose(fp);
+			close(fd);
+			fatal("Cannot move tasks out of cpuset %s\n", name);
+		}
+
+
 	}
 	fclose(fp);
 	close(fd);
@@ -655,19 +558,24 @@ static void destroy_cpuset(const char *name, int print)
 static void teardown(void)
 {
 	int fd;
+	int ret;
 
 	if (all_cpus)
 		return;
 
 	fd = open_cpuset(CPUSET_PATH, "cpuset.cpu_exclusive");
 	if (fd >= 0) {
-		write(fd, "0", 2);
+		ret = write(fd, "0", 2);
+		if (ret < 0)
+			perror("cpuset.cpu_exclusive");
 		close(fd);
 	}
 
 	fd = open_cpuset(CPUSET_PATH, "cpuset.sched_load_balance");
 	if (fd >= 0) {
-		write(fd, "1", 2);
+		ret = write(fd, "1", 2);
+		if (ret < 0)
+			perror("cpuset.sched_load_balance");
 		close(fd);
 	}
 
@@ -687,10 +595,14 @@ static void usage(int error)
 	       "-a [CPUSET] --affinity     Comma/hyphen separated list of CPUs to run deadline\n"
 	       "                           tasks on. An empty CPUSET runs on all CPUs a deadline\n"
 	       "                           task.\n"
-	       "-D TIME  --duration        Specify a length for the test run.\n"
+	       "-D TIME  --duration        Specify a length in seconds for the test run.\n"
 	       "                           Append 'm', 'h', or 'd' to specify minutes, hours or\n"
 	       "                           days\n"
 	       "-h       --help            Show this help menu.\n"
+	       "         --histogram=US    dump a latency histogram to stdout after the run\n"
+	       "                           US is the max latency time to be tracked in microseconds\n"
+	       "			   This option runs all threads at the same priority.\n"
+	       "	 --histfile=<path> dump the latency histogram to <path> instead of stdout\n"
 	       "-i INTV  --interval        The shortest deadline for the tasks in us\n"
 	       "                           (default 1000us).\n"
 	       "         --json=FILENAME   write final results into FILENAME, JSON formatted\n"
@@ -718,39 +630,74 @@ static u64 get_time_us(void)
 	return time;
 }
 
-static void print_stat(FILE *fp, struct sched_data *sd, int index, int verbose, int quiet)
+static void print_hist(FILE *fp, struct sched_data *sd, int nthreads)
 {
-	struct thread_stat *stat = &sd->stat;
+	int i;
+	unsigned long maxmax, alloverflows;
 
-	if (!verbose) {
-		if (quiet != 1) {
-			char *fmt;
-			if (use_nsecs)
-				fmt = "T:%2d (%5d) I:%ld C:%7lu "
-					"Min:%7ld Act:%8ld Avg:%8ld Max:%8ld\n";
-			else
-				fmt = "T:%2d (%5d) I:%ld C:%7lu "
-					"Min:%7ld Act:%5ld Avg:%5ld Max:%8ld\n";
-			fprintf(fp, fmt, index, stat->tid,
-				sd->deadline_us, stat->cycles, stat->min, stat->act,
-				stat->cycles ?
-				(long)(stat->avg/stat->cycles) : 0, stat->max);
-		}
-	} else {
-		while (stat->cycles != stat->cyclesread) {
-			long diff = stat->values
-			    [stat->cyclesread & sd->bufmsk];
+	fprintf(fp, "# Histogram\n");
+	for (i = 0; i < histogram; i++) {
+		unsigned long flags = 0;
+		char buf[64];
 
-			if (diff > stat->redmax) {
-				stat->redmax = diff;
-				stat->cycleofmax = stat->cyclesread;
-			}
-			stat->cyclesread++;
-		}
+		snprintf(buf, sizeof(buf), "%06d ", i);
+		hset_print_bucket(&hset, fp, buf, i, flags);
 	}
+	fprintf(fp, "# Min Latencies:");
+	for (i = 0; i < nthreads; i++)
+		fprintf(fp, " %05lu", sd[i].stat.min);
+	fprintf(fp, "\n");
+	fprintf(fp, "# Avg Latencies:");
+	for (i = 0; i < nthreads; i++)
+		fprintf(fp, " %05lu", sd[i].stat.cycles ?
+		       (long)(sd[i].stat.avg/sd[i].stat.cycles) : 0);
+	fprintf(fp, "\n");
+	fprintf(fp, "# Max Latencies:");
+	maxmax = 0;
+	for (i = 0; i < nthreads; i++) {
+		fprintf(fp, " %05lu", sd[i].stat.max);
+		if (sd[i].stat.max > maxmax)
+			maxmax = sd[i].stat.max;
+	}
+	fprintf(fp, "\n");
+	fprintf(fp, "# Histogram Overflows:");
+	alloverflows = 0;
+	for (i = 0; i < nthreads; i++) {
+		fprintf(fp, " %05lu", sd[i].stat.hist->oflow_count);
+		alloverflows += sd[i].stat.hist->oflow_count;
+	}
+	fprintf(fp, "\n");
+
+	fprintf(fp, "# Histogram Overflow at cycle number:\n");
+	for (i = 0; i < nthreads; i++) {
+		fprintf(fp, "# Thread %d: ", i);
+		hist_print_oflows(sd[i].stat.hist, fp);
+		fprintf(fp, "\n");
+	}
+
+	fprintf(fp, "\n");
 }
 
-static u64 do_runtime(long tid, struct sched_data *sd, u64 period)
+static void print_stat(FILE *fp, struct sched_data *sd, int index, int quiet)
+{
+	struct thread_stat *stat = &sd->stat;
+	char *fmt;
+
+	if (quiet)
+		return;
+
+	if (use_nsecs)
+		fmt = "T:%2d (%5d) I:%ld C:%7lu Min:%7ld Act:%8ld Avg:%8ld Max:%8ld\n";
+	else
+		fmt = "T:%2d (%5d) I:%ld C:%7lu Min:%7ld Act:%5ld Avg:%5ld Max:%8ld\n";
+
+	fprintf(fp, fmt, index, stat->tid,
+		sd->deadline_us, stat->cycles, stat->min, stat->act,
+		stat->cycles ?
+		(long)(stat->avg/stat->cycles) : 0, stat->max);
+}
+
+static u64 do_runtime(struct sched_data *sd, u64 period)
 {
 	struct thread_stat *stat = &sd->stat;
 	u64 next_period = period + sd->deadline_us;
@@ -764,16 +711,15 @@ static u64 do_runtime(long tid, struct sched_data *sd, u64 period)
 		 * preempting us when we started. If that's the case then
 		 * adjust the current period.
 		 */
-		ftrace_write(sd->buff,
-			     "Adjusting period: now: %lld period: %lld delta:%lld%s\n",
-			     now, period, delta, delta > sd->deadline_us / 2 ?
-			     " HUGE ADJUSTMENT" : "");
+		tracemark("Adjusting period: now: %lld period: %lld delta:%lld%s\n",
+			  now, period, delta, delta > sd->deadline_us / 2 ?
+			  " HUGE ADJUSTMENT" : "");
 		period = now;
 		next_period = period + sd->deadline_us;
 	}
 
-	ftrace_write(sd->buff, "start at %lld off=%lld (period=%lld next=%lld)\n",
-		     now, now - period, period, next_period);
+	tracemark("start at %lld off=%lld (period=%lld next=%lld)\n",
+		  now, now - period, period, next_period);
 
 
 	diff = now - period;
@@ -784,6 +730,9 @@ static u64 do_runtime(long tid, struct sched_data *sd, u64 period)
 	stat->act = diff;
 	stat->avg += (double) diff;
 
+	if (histogram)
+		hist_sample(stat->hist, diff);
+
 	stat->cycles++;
 
 	return next_period;
@@ -833,7 +782,7 @@ void *run_deadline(void *data)
 	period = get_time_us();
 
 	while (!shutdown) {
-		period = do_runtime(tid, sd, period);
+		period = do_runtime(sd, period);
 		if (tracelimit && (stat->max > tracelimit)) {
 			shutdown++;
 			pthread_mutex_lock(&break_thread_id_lock);
@@ -842,6 +791,7 @@ void *run_deadline(void *data)
 				break_thread_value = stat->max;
 				tracemark("hit latency threshold (%lld > %d)",
 						 (unsigned long long) stat->max, tracelimit);
+				tracing_stop();
 			}
 			pthread_mutex_unlock(&break_thread_id_lock);
 			break;
@@ -1030,7 +980,7 @@ static int calc_nr_cpus(const char *setcpu, char **buf)
 	return -1;
 }
 
-static void sighand(int sig __attribute__ ((unused)))
+static void sighand(int sig)
 {
 	shutdown = 1;
 }
@@ -1049,7 +999,7 @@ static void loop(struct sched_data *sched_data, int nr_threads)
 
 	while (!shutdown) {
 		for (i = 0; i < nr_threads; i++)
-			print_stat(stdout, &sched_data[i], i, 0, quiet);
+			print_stat(stdout, &sched_data[i], i, quiet);
 		usleep(10000);
 		if (!quiet)
 			printf("\033[%dA", nr_threads);
@@ -1057,9 +1007,15 @@ static void loop(struct sched_data *sched_data, int nr_threads)
 	usleep(10000);
 	if (!quiet) {
 		printf("\033[%dB", nr_threads + 2);
-	} else {
+	} else if (!histogram) {
 		for (i = 0; i < nr_threads; ++i)
-			print_stat(stdout, &sched_data[i], i, 0, 0);
+			print_stat(stdout, &sched_data[i], i, 0);
+	}
+
+	if (histogram) {
+		FILE *out = histfile ? histfile : stdout;
+
+		print_hist(out, sched_data, nr_threads);
 	}
 }
 
@@ -1075,10 +1031,15 @@ static void write_stats(FILE *f, void *data)
 	for (i = 0; i < nr_threads; i++) {
 		s = &sd[i].stat;
 		fprintf(f, "    \"%u\": {\n", i);
-		fprintf(f, "	 \"cycles\": %ld,\n", s->cycles);
-		fprintf(f, "	 \"min\": %ld,\n", s->min);
-		fprintf(f, "	 \"max\": %ld,\n", s->max);
-		fprintf(f, "	 \"avg\": %.2f\n", s->avg/s->cycles);
+		if (s->hist) {
+			fprintf(f, "      \"histogram\": {");
+			hist_print_json(s->hist, f);
+			fprintf(f, "      },\n");
+		}
+		fprintf(f, "      \"cycles\": %ld,\n", s->cycles);
+		fprintf(f, "      \"min\": %ld,\n", s->min);
+		fprintf(f, "      \"max\": %ld,\n", s->max);
+		fprintf(f, "      \"avg\": %.2f\n", s->avg/s->cycles);
 		fprintf(f, "    }%s\n", i == nr_threads - 1 ? "" : ",");
 	}
 	fprintf(f, "  }\n");
@@ -1088,6 +1049,7 @@ enum options_values {
 	OPT_AFFINITY=1, OPT_DURATION, OPT_HELP, OPT_INTERVAL,
 	OPT_JSON, OPT_STEP, OPT_THREADS, OPT_QUIET,
 	OPT_BREAKTRACE, OPT_TRACEMARK, OPT_INFO, OPT_DEBUG,
+	OPT_HISTOGRAM, OPT_HISTFILE
 };
 
 int main(int argc, char **argv)
@@ -1109,6 +1071,7 @@ int main(int argc, char **argv)
 	int nr_cpus;
 	int i;
 	int c;
+	int ret;
 
 	rt_init(argc, argv);
 
@@ -1130,6 +1093,8 @@ int main(int argc, char **argv)
 			{ "tracemark",	     no_argument,	NULL, OPT_TRACEMARK },
 			{ "verbose",	no_argument,	NULL,	OPT_INFO},
 			{ "debug",	no_argument, 	NULL, 	OPT_DEBUG},
+			{ "histogram",	required_argument, NULL, OPT_HISTOGRAM },
+			{ "histfile",	required_argument, NULL, OPT_HISTFILE },
 			{ NULL,		0,			NULL,	0   },
 		};
 		c = getopt_long(argc, argv, "a::c:D:hi:s:t:b:q", options, NULL);
@@ -1188,6 +1153,17 @@ int main(int argc, char **argv)
 		case OPT_DEBUG:
 			debug_enable = 1;
 			break;
+		case OPT_HISTOGRAM:
+			histogram = atoi(optarg);
+			if (histogram <= 0 || histogram > HIST_MAX)
+				usage(1);
+			break;
+		case OPT_HISTFILE:
+			histfile = fopen(optarg, "w");
+			if (!histfile)
+				fatal("Couldn\'t open histfile %s: %s\n",
+				      optarg, strerror(errno));
+			break;
 		default:
 			usage(1);
 		}
@@ -1224,7 +1200,6 @@ int main(int argc, char **argv)
 	if (mlockall(MCL_CURRENT|MCL_FUTURE) == -1)
 		warn("mlockall");
 
-	setup_ftrace_marker();
 	if (tracelimit && trace_marker)
 		enable_trace_mark();
 
@@ -1233,6 +1208,10 @@ int main(int argc, char **argv)
 	if (!thread || !sched_data)
 		fatal("allocating threads");
 
+	if (histogram && hset_init(&hset, nr_threads, 1, histogram, histogram))
+		fatal("failed to allocate histogram of size %d for %d threads\n",
+		      histogram, nr_threads);
+
 	if (nr_threads > nr_cpus) {
 		/*
 		 * More threads than CPUs, then have the total be
@@ -1262,11 +1241,14 @@ int main(int argc, char **argv)
 		sd->runtime_us = runtime;
 		sd->deadline_us = interval;
 
+		if (histogram)
+			sd->stat.hist = &hset.histos[i];
+
 		info(info_enable, "interval: %lld:%lld\n", sd->runtime_us, sd->deadline_us);
 
 		/* Make sure that we can make our deadlines */
 		start_period = get_time_us();
-		do_runtime(gettid(), sd, start_period);
+		do_runtime(sd, start_period);
 		end_period = get_time_us();
 		if (end_period - start_period > sd->runtime_us)
 			fatal("Failed to perform task within runtime: Missed by %lld us\n",
@@ -1312,7 +1294,9 @@ int main(int argc, char **argv)
 				  CPUSET_FL_CLONE_CHILDREN |
 				  CPUSET_FL_TASKS, pids);
 
-		system("cat /sys/fs/cgroup/cpuset/my_cpuset/tasks");
+		ret = system("cat /sys/fs/cgroup/cpuset/my_cpuset/tasks");
+		if (ret < 0)
+			perror("system call failed");
 	}
 
 	debug(debug_enable, "main thread %d\n", gettid());
@@ -1356,6 +1340,9 @@ int main(int argc, char **argv)
 		free(setcpu_buf);
 	free(thread);
 	free(sched_data);
+	if (histfile)
+		fclose(histfile);
+	hset_destroy(&hset);
 
 	return 0;
 }
